@@ -5,3 +5,8 @@ from boring.server import Server
 
 def main():
     Server().start()
+
+
+if __name__ == '__main__':
+    # TODO
+    pass
