@@ -1,7 +1,7 @@
 import io
 import time
 import urllib.parse
-
+import socket
 from boring import __version__
 from boring.exception import BadRequest, InvalidHeader
 from boring.utils import http_date
@@ -58,7 +58,7 @@ class Response:
         self.req = req
         self.conn = conn
         self.headers = []
-        self.code = ""
+        self.code = None
         self.reason = ""
         self.sent = 0
         self.headers_set = False
@@ -117,16 +117,15 @@ class Response:
         ]
         status = b" ".join(status)
         self.write(status)
-        #self.conn.send(status)
         connection = self.req.headers.get('Connection', 'close')
         self.headers.append(("Connection", connection))
         header = self.process_headers(self.headers)
-        #self.write_body(h)
         self.write(header)
         self.headers_sent = True
         self.write_body(data, size, chunck)
 
     def write_chunck(self, data):
+        ''' send the data with chunck transfer'''
         iterator = iter(data)
         try:
             while 1:
@@ -154,6 +153,8 @@ class Response:
                     data_chunck = next(iterator)
                 except StopIteration:
                     break
+                # make sure the server doesnt send data
+                # more than the content-length specified
                 if self.sent >= size:
                     return
                 remain = size - self.sent
@@ -184,7 +185,7 @@ class BodyReader:
 
     @property
     def size(self):
-        return self.tell
+        return self.tell()
 
     def seek(self, pos):
         self.buf.seek(pos)
@@ -248,11 +249,28 @@ class HTTPParser:
                 self.begin = True
                 return
             chunck = buf.readline(size)
-            if len(chunck) != size:
+            if len(chunck) < size:
                 while len(chunck) < size:
-                    # This fixes chunck  length less than size
-                    # caused by .readline()
-                    chunck += buf.readline(size - len(chunck))
+                    # either all the data in buf has been read
+                    # or ,readline() doesnt return required amount
+                    remain = buf.readline(size - len(chunck))
+                    if not remain:
+                        #   all the data in buf has been read
+                        #   get the current position
+                        pos = buf.tell()
+                        self.conn.settimeout(0.5)
+                        try:
+                            #  recieve remaining data from the client
+                            data = self.conn.recv(1024)
+                        except (socket.error, socket.timeout):
+                            return
+                        #  write it into the buf
+                        buf.write(data)
+                        #  seek to the current position before
+                        # writing new data
+                        buf.seek(pos)
+                        continue
+                    chunck += remain
             buf.readline()  # discard trailing \r\n
             self.body.write(chunck)
 
@@ -281,7 +299,8 @@ class HTTPParser:
             # self.body_set = True
 
     def __call__(self, conn=None):
-        data = self.conn.recv(1024)
+        data = self.conn.recv(2048)
+        # print(data.decode())
         if not data:
             #client close connection
             self.is_alive = False
