@@ -1,12 +1,11 @@
-import argparse
-import errno
-import importlib
+import contextlib
 import logging
 import os
 import selectors
 import signal
 import socket
 import sys
+import time
 import threading
 import traceback
 
@@ -16,18 +15,22 @@ from boring.exception import HttpException
 from boring.http import HTTPParser, Request
 from boring.wsgi import WsgiApp
 
+from .dir import DirectoryServer
+from . import reloader
+
 
 class Logger:
-    # [23/Jan/2021 12:43:30] code 501, message Unsupported method ('POST')
+
+    # 127.0.0.1 -- [01/07/2021 21:09:48 PM] GET / HTTP/1.1 200 --
     def __init__(self):
-        self._access = logging.getLogger(__name__)
+        self._access = logging.getLogger('boring.server')
         self._access.setLevel(logging.DEBUG)
         ch = logging.StreamHandler()
         ch.setLevel(logging.DEBUG)
 
         formatter = logging.Formatter(
-            ' %(addr)s -- [%(asctime)s] %(method)s %(path)s %(proto)s %(code)s -- %(message)s'
-        )
+            ' %(addr)s -- [%(asctime)s] %(method)s %(path)s %(proto)s %(code)s -- %(message)s',
+            datefmt='%d/%m/%Y %H:%M:%S %p')
         ch.setFormatter(formatter)
         ##
         ### add ch to logger
@@ -78,16 +81,19 @@ class SignalHandler:
 
 
 class Server:
-    def __init__(self, config=None):
+    def __init__(self, app=None, config=None, args=None):
         self.sel = selectors.DefaultSelector()
         self.sock = socket.socket()
         self.signals = ['SIGTERM', "SIGINT"]
         self.signal_class = SignalHandler(self)
-        self.app = None
+        self.module = None
+        self.app = app
         self.args = None
         self.log = Logger()
         self.stop = False
         self.config = config or DummyConfig()
+        self._active_conns = {}
+        self.started = False
 
     def init_socket(self):
         port = self.args.port
@@ -96,7 +102,7 @@ class Server:
             self.sock.bind((addr, int(port)))
         except OSError as e:
             print("[ERROR] could't bind to address %s:%s" % (addr, port), e)
-            sys.exit(-1)
+            sys.exit(1)
         self.sock.listen(100)
         self.sock.setblocking(False)
         self.sel.register(self.sock, selectors.EVENT_READ,
@@ -108,7 +114,9 @@ class Server:
         self.init()
         self.init_socket()
         print("[INFO]", 'server started, press control-c to stop')
-        self.start_reload()
+        self.started = True
+        if "BORING_RELOAD_PROC" in os.environ:
+            reloader.start(self)
         while 1:
             events = self.sel.select(0)
             for key, _ in events:
@@ -124,6 +132,7 @@ class Server:
                         traceback.print_exc()
                         self.close_connection(key.fileobj)
                         #key.fileobj.send(b"h")
+            self.manage_connections()
 
     def init_signals(self):
         for sig in self.signals:
@@ -138,21 +147,34 @@ class Server:
 
     def init(self):
         self.init_signals()
-        self.args = utils.create_args()
+        args = self.create_args()
+        self.check_reload_arg()
         if self.args.use_config:
             self.config = Config(self.args)
             self.config.load()
-        self.load_app()
+        if self.args.app == ".":
+            self.module = DirectoryServer
+        else:
+            self.load_app()
         # self.start_reload()
 
-    def start_reload(self):
-        if self.args.reload:
-            threading.Thread(target=reload, args=[self]).start()
-            print('[INFO] auto reload starting')
+    def check_reload_arg(self):
+        if not self.args.reload:
+            return
+        reload_proc = os.environ.get("BORING_RELOAD_PROC")
+        if not reload_proc:
+            # main process
+            code = reloader.start_new_process()
+            sys.exit(code)
 
     def load_app(self):
         self.app = wsgi.load_app(self.args)
 
+    def create_args(self):
+        args = utils.create_args()
+        self.args = args
+        return args
+
     def handle_connection(self, sock):
         try:
             conn, addr = sock.accept()
@@ -162,6 +184,7 @@ class Server:
         self.sel.register(conn,
                           selectors.EVENT_READ,
                           data=HTTPParser(conn, self, addr))
+        self._active_conns[conn] = int(time.time())
         #self.sel.unregister(conn)
 
     def handle_request(self, conn, parser):
@@ -189,9 +212,7 @@ class Server:
             return
         try:
             request = Request(parser)
-            wsgiapp = WsgiApp(self.app, request, conn, self, self.log,
-                              self.config)
-            wsgiapp.dispatch_request()
+            self.run(request, conn)
         except socket.error:
             self.close_connection(conn)
             return
@@ -202,10 +223,16 @@ class Server:
             self.close_connection(conn)
             traceback.print_exc()
 
-    def handle_filechange(self, file):
-        ''' this function is called from another thread when files change'''
-        importlib.reload(file)
-        self.app = wsgi.reload_app(self.args)
+    def run(self, request, conn):
+        if self.module:
+            self.module(conn, request, self).run()
+            self.close_connection(conn)
+            return
+        wsgiapp = WsgiApp(self.app, request, conn, self.log, self, self.config)
+        wsgiapp.run()
+        # wsgiapp = WsgiApp(self.app, request, conn, self, self.log,
+        #                       self.config)
+        #     wsgiapp.dispatch_request()
 
     def close_connection(self, conn):
         ''' Close the connection after serving the request '''
@@ -215,21 +242,34 @@ class Server:
             self.sel.unregister(conn)
         except (ValueError, KeyError):  # raised by selectors.unregister
             pass
-        finally:
+        try:
             conn.close()
+        except OSError:
+            pass
+        with contextlib.suppress(KeyError):
+            del self._active_conns[conn]
 
     def reuse_connection(self, conn, req):
         """ re-use connection for keep-alive header"""
-        self.close_connection(conn)
-        return
-        # # dont want to keep connection for now
-        # try:
-        #     self.sel.unregister(conn)
-        #     self.sel.register(conn,
-        #                       selectors.EVENT_READ,
-        #                       data=HTTPParser(conn, self, req.addr))
-        # except (KeyError, ValueError):
-        #     pass
+        try:
+            self.sel.unregister(conn)
+            self.sel.register(conn,
+                              selectors.EVENT_READ,
+                              data=HTTPParser(conn, self, req.addr))
+        except (KeyError, ValueError):
+            self.close_connection(conn)
+        else:
+            self._active_conns[conn] = int(time.time())
+
+    def manage_connections(self):
+        '''  remove some connections that are inactive for some time'''
+        active = self._active_conns.items()
+        timeout_conns = []
+        for conn, conn_time in active:
+            if int(time.time()) - conn_time > 30:
+                timeout_conns.append(conn)
+        for conn in timeout_conns:
+            self.close_connection(conn)
 
     def shutdown(self):
         ''' shutdown the server'''
@@ -237,24 +277,3 @@ class Server:
         self.sel.close()
         self.stop = True
         sys.exit(0)
-
-
-def reload(server):
-    mtimes = {}
-    for module_name, module in sys.modules.items():
-        if hasattr(module, '__file__') and module.__file__ is not None:
-            mtime = os.stat(module.__file__).st_mtime
-            mtimes[module] = mtime
-    while 1:
-        for mod in mtimes:
-            if server.stop:
-                sys.exit(0)
-            s = os.stat(mod.__file__).st_mtime
-            if s > mtimes[mod]:
-                print(mod.__file__, 'changed')
-                mtimes[mod] = s
-                try:
-                    server.handle_filechange(mod)
-                except Exception:
-                    print("[CRITICAL] error loading file", mod.__file__)
-                    traceback.print_exc()
